@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2013 Conrad Sanderson
-// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup Cube
@@ -27,27 +29,29 @@ class Cube : public BaseCube< eT, Cube<eT> >
   public:
   
   typedef eT                                elem_type; //!< the type of elements stored in the cube
-  typedef typename get_pod_type<eT>::result pod_type;  //!< if eT is non-complex, pod_type is same as eT. otherwise, pod_type is the underlying type used by std::complex
+  typedef typename get_pod_type<eT>::result  pod_type; //!< if eT is std::complex<T>, pod_type is T; otherwise pod_type is eT
   
-  const uword  n_rows;       //!< number of rows in each slice (read-only)
-  const uword  n_cols;       //!< number of columns in each slice (read-only)
+  const uword  n_rows;       //!< number of rows     in each slice (read-only)
+  const uword  n_cols;       //!< number of columns  in each slice (read-only)
   const uword  n_elem_slice; //!< number of elements in each slice (read-only)
-  const uword  n_slices;     //!< number of slices in the cube (read-only)
-  const uword  n_elem;       //!< number of elements in the cube (read-only)
+  const uword  n_slices;     //!< number of slices   in the cube   (read-only)
+  const uword  n_elem;       //!< number of elements in the cube   (read-only)
   const uword  mem_state;
   
-  // mem_state = 0: normal cube that can be resized; 
-  // mem_state = 1: use auxiliary memory until change in the number of elements is requested;  
-  // mem_state = 2: use auxiliary memory and don't allow the number of elements to be changed; 
-  // mem_state = 3: fixed size (e.g. via template based size specification).
+  // mem_state = 0: normal cube which manages its own memory
+  // mem_state = 1: use auxiliary memory until a size change
+  // mem_state = 2: use auxiliary memory and don't allow the number of elements to be changed
+  // mem_state = 3: fixed size (eg. via template based size specification)
   
+  arma_aligned const eT* const mem;  //!< pointer to the memory used for storing elements (memory is read-only)
   
-  arma_aligned const Mat<eT>** const mat_ptrs; //!< pointer to an array containing pointers to Mat instances (one for each slice)
-  arma_aligned const eT*       const mem;      //!< pointer to the memory used by the cube (memory is read-only)
   
   protected:
+  
+  arma_aligned const Mat<eT>** const mat_ptrs;
+  
   arma_align_mem Mat<eT>* mat_ptrs_local[ Cube_prealloc::mat_ptrs_size ];
-  arma_align_mem eT            mem_local[ Cube_prealloc::mem_n_elem    ];
+  arma_align_mem eT            mem_local[ Cube_prealloc::mem_n_elem    ];  // local storage, for small cubes
   
   
   public:
@@ -55,17 +59,18 @@ class Cube : public BaseCube< eT, Cube<eT> >
   inline ~Cube();
   inline  Cube();
   
-  inline          Cube(const uword in_rows, const uword in_cols, const uword in_slices);
+  inline explicit Cube(const uword in_rows, const uword in_cols, const uword in_slices);
+  inline explicit Cube(const SizeCube& s);
   
-  template<typename fill_type>
-  inline Cube(const uword in_rows, const uword in_cols, const uword in_slices, const fill::fill_class<fill_type>& f);
+  template<typename fill_type> inline Cube(const uword in_rows, const uword in_cols, const uword in_slices, const fill::fill_class<fill_type>& f);
+  template<typename fill_type> inline Cube(const SizeCube& s,                                               const fill::fill_class<fill_type>& f);
   
   #if defined(ARMA_USE_CXX11)
   inline                  Cube(Cube&& m);
   inline const Cube& operator=(Cube&& m);
   #endif
   
-  inline Cube(      eT* aux_mem, const uword aux_n_rows, const uword aux_n_cols, const uword aux_n_slices, const bool copy_aux_mem = true, const bool strict = true);
+  inline Cube(      eT* aux_mem, const uword aux_n_rows, const uword aux_n_cols, const uword aux_n_slices, const bool copy_aux_mem = true, const bool strict = false, const bool prealloc_mat = false);
   inline Cube(const eT* aux_mem, const uword aux_n_rows, const uword aux_n_cols, const uword aux_n_slices);
   
   arma_inline const Cube&  operator=(const eT val);
@@ -131,6 +136,18 @@ class Cube : public BaseCube< eT, Cube<eT> >
   template<typename T1> arma_inline const subview_elem1<eT,T1> operator()(const Base<uword,T1>& a) const;
   
   
+  arma_inline       subview_cube_each1<eT> each_slice();
+  arma_inline const subview_cube_each1<eT> each_slice() const;
+  
+  template<typename T1> inline       subview_cube_each2<eT, T1> each_slice(const Base<uword, T1>& indices);
+  template<typename T1> inline const subview_cube_each2<eT, T1> each_slice(const Base<uword, T1>& indices) const;
+  
+  #if defined(ARMA_USE_CXX11)
+  inline const Cube& each_slice(const std::function< void(      Mat<eT>&) >& F);
+  inline const Cube& each_slice(const std::function< void(const Mat<eT>&) >& F) const;
+  #endif
+  
+  
   inline void shed_slice(const uword slice_num);
   
   inline void shed_slices(const uword in_slice1, const uword in_slice2);
@@ -217,6 +234,9 @@ class Cube : public BaseCube< eT, Cube<eT> >
   arma_inline arma_warn_unused bool is_finite() const;
   arma_inline arma_warn_unused bool is_empty()  const;
   
+  inline arma_warn_unused bool has_inf() const;
+  inline arma_warn_unused bool has_nan() const;
+  
   arma_inline arma_warn_unused bool in_range(const uword i) const;
   arma_inline arma_warn_unused bool in_range(const span& x) const;
   
@@ -234,39 +254,45 @@ class Cube : public BaseCube< eT, Cube<eT> >
   arma_inline arma_warn_unused       eT* slice_colptr(const uword in_slice, const uword in_col);
   arma_inline arma_warn_unused const eT* slice_colptr(const uword in_slice, const uword in_col) const;
   
-  inline void impl_print(const std::string& extra_text) const;
+  inline void impl_print(                           const std::string& extra_text) const;
   inline void impl_print(std::ostream& user_stream, const std::string& extra_text) const;
   
-  inline void impl_raw_print(const std::string& extra_text) const;
+  inline void impl_raw_print(                           const std::string& extra_text) const;
   inline void impl_raw_print(std::ostream& user_stream, const std::string& extra_text) const;
   
   inline void  set_size(const uword in_rows, const uword in_cols, const uword in_slices);
   inline void   reshape(const uword in_rows, const uword in_cols, const uword in_slices, const uword dim = 0);
   inline void    resize(const uword in_rows, const uword in_cols, const uword in_slices);
   
+  inline void  set_size(const SizeCube& s);
+  inline void   reshape(const SizeCube& s);
+  inline void    resize(const SizeCube& s);
+  
   template<typename eT2> inline void copy_size(const Cube<eT2>& m);
   
+  template<typename functor> inline const Cube&  for_each(functor F);
+  template<typename functor> inline const Cube&  for_each(functor F) const;
   
-  template<typename functor>
-  inline const Cube& transform(functor F);
-  
-  template<typename functor>
-  inline const Cube& imbue(functor F);
-  
+  template<typename functor> inline const Cube& transform(functor F);
+  template<typename functor> inline const Cube&     imbue(functor F);
   
   inline const Cube& fill(const eT val);
   
   inline const Cube& zeros();
   inline const Cube& zeros(const uword in_rows, const uword in_cols, const uword in_slices);
+  inline const Cube& zeros(const SizeCube& s);
   
   inline const Cube& ones();
   inline const Cube& ones(const uword in_rows, const uword in_cols, const uword in_slices);
+  inline const Cube& ones(const SizeCube& s);
   
   inline const Cube& randu();
   inline const Cube& randu(const uword in_rows, const uword in_cols, const uword in_slices);
+  inline const Cube& randu(const SizeCube& s);
   
   inline const Cube& randn();
   inline const Cube& randn(const uword in_rows, const uword in_cols, const uword in_slices);
+  inline const Cube& randn(const SizeCube& s);
   
   inline void reset();
   
@@ -345,6 +371,7 @@ class Cube : public BaseCube< eT, Cube<eT> >
   friend class glue_join;
   friend class op_reshape;
   friend class op_resize;
+  friend class subview_cube<eT>;
   
   
   public:
