@@ -1,10 +1,12 @@
-// Copyright (C) 2008-2013 Conrad Sanderson
-// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
-// Copyright (C) 2012 Ryan Curtin
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Written by Ryan Curtin
 
 
 //! \addtogroup Mat
@@ -20,23 +22,25 @@ class Mat : public Base< eT, Mat<eT> >
   public:
   
   typedef eT                                elem_type;  //!< the type of elements stored in the matrix
-  typedef typename get_pod_type<eT>::result pod_type;   //!< if eT is non-complex, pod_type is same as eT. otherwise, pod_type is the underlying type used by std::complex
+  typedef typename get_pod_type<eT>::result  pod_type;  //!< if eT is std::complex<T>, pod_type is T; otherwise pod_type is eT
   
-  const uword  n_rows;    //!< number of rows in the matrix (read-only)
-  const uword  n_cols;    //!< number of columns in the matrix (read-only)
-  const uword  n_elem;    //!< number of elements in the matrix (read-only)
+  const uword  n_rows;    //!< number of rows     (read-only)
+  const uword  n_cols;    //!< number of columns  (read-only)
+  const uword  n_elem;    //!< number of elements (read-only)
   const uhword vec_state; //!< 0: matrix layout; 1: column vector layout; 2: row vector layout
   const uhword mem_state;
   
-  // mem_state = 0: normal matrix that can be resized; 
-  // mem_state = 1: use auxiliary memory until change in the number of elements is requested;  
-  // mem_state = 2: use auxiliary memory and don't allow the number of elements to be changed; 
-  // mem_state = 3: fixed size (e.g. via template based size specification).
+  // mem_state = 0: normal matrix which manages its own memory
+  // mem_state = 1: use auxiliary memory until a size change
+  // mem_state = 2: use auxiliary memory and don't allow the number of elements to be changed
+  // mem_state = 3: fixed size (eg. via template based size specification)
+  
+  arma_aligned const eT* const mem;  //!< pointer to the memory used for storing elements (memory is read-only)
   
-  arma_aligned const eT* const mem;  //!< pointer to the memory used by the matrix (memory is read-only)
   
   protected:
-  arma_align_mem eT mem_local[ arma_config::mat_prealloc ];
+  
+  arma_align_mem eT mem_local[ arma_config::mat_prealloc ];  // local storage, for small vectors and matrices
   
   
   public:
@@ -47,10 +51,11 @@ class Mat : public Base< eT, Mat<eT> >
   inline ~Mat();
   inline  Mat();
   
-  inline Mat(const uword in_rows, const uword in_cols);
+  inline explicit Mat(const uword in_rows, const uword in_cols);
+  inline explicit Mat(const SizeMat& s);
   
-  template<typename fill_type>
-  inline Mat(const uword in_rows, const uword in_cols, const fill::fill_class<fill_type>& f);
+  template<typename fill_type> inline Mat(const uword in_rows, const uword in_cols, const fill::fill_class<fill_type>& f);
+  template<typename fill_type> inline Mat(const SizeMat& s,                         const fill::fill_class<fill_type>& f);
   
   inline                  Mat(const char*        text);
   inline const Mat& operator=(const char*        text);
@@ -65,11 +70,14 @@ class Mat : public Base< eT, Mat<eT> >
   inline                  Mat(const std::initializer_list<eT>& list);
   inline const Mat& operator=(const std::initializer_list<eT>& list);
   
+  inline                  Mat(const std::initializer_list< std::initializer_list<eT> >& list);
+  inline const Mat& operator=(const std::initializer_list< std::initializer_list<eT> >& list);
+  
   inline                  Mat(Mat&& m);
   inline const Mat& operator=(Mat&& m);
   #endif
   
-  inline Mat(      eT* aux_mem, const uword aux_n_rows, const uword aux_n_cols, const bool copy_aux_mem = true, const bool strict = true);
+  inline Mat(      eT* aux_mem, const uword aux_n_rows, const uword aux_n_cols, const bool copy_aux_mem = true, const bool strict = false);
   inline Mat(const eT* aux_mem, const uword aux_n_rows, const uword aux_n_cols);
   
   arma_inline const Mat&  operator=(const eT val);
@@ -109,7 +117,9 @@ class Mat : public Base< eT, Mat<eT> >
   inline Mat(const subview_row_htrans<eT>& X);  // subview_row_htrans can only be generated by the Proxy class
   inline Mat(const        xvec_htrans<eT>& X);  //        xvec_htrans can only be generated by the Proxy class
   
-  //inline explicit          Mat(const subview_cube<eT>& X);
+  template<bool do_conj>
+  inline Mat(const xtrans_mat<eT,do_conj>& X);  //        xtrans_mat can only be generated by the Proxy class
+  
   inline                   Mat(const subview_cube<eT>& X);
   inline const Mat&  operator=(const subview_cube<eT>& X);
   inline const Mat& operator+=(const subview_cube<eT>& X);
@@ -117,8 +127,7 @@ class Mat : public Base< eT, Mat<eT> >
   inline const Mat& operator*=(const subview_cube<eT>& X);
   inline const Mat& operator%=(const subview_cube<eT>& X);
   inline const Mat& operator/=(const subview_cube<eT>& X);
-
-  //inline explicit          Mat(const diagview<eT>& X);
+  
   inline                   Mat(const diagview<eT>& X);
   inline const Mat&  operator=(const diagview<eT>& X);
   inline const Mat& operator+=(const diagview<eT>& X);
@@ -127,6 +136,14 @@ class Mat : public Base< eT, Mat<eT> >
   inline const Mat& operator%=(const diagview<eT>& X);
   inline const Mat& operator/=(const diagview<eT>& X);
   
+  inline                   Mat(const spdiagview<eT>& X);
+  inline const Mat&  operator=(const spdiagview<eT>& X);
+  inline const Mat& operator+=(const spdiagview<eT>& X);
+  inline const Mat& operator-=(const spdiagview<eT>& X);
+  inline const Mat& operator*=(const spdiagview<eT>& X);
+  inline const Mat& operator%=(const spdiagview<eT>& X);
+  inline const Mat& operator/=(const spdiagview<eT>& X);
+  
   template<typename T1> inline                   Mat(const subview_elem1<eT,T1>& X);
   template<typename T1> inline const Mat& operator= (const subview_elem1<eT,T1>& X);
   template<typename T1> inline const Mat& operator+=(const subview_elem1<eT,T1>& X);
@@ -142,8 +159,8 @@ class Mat : public Base< eT, Mat<eT> >
   template<typename T1, typename T2> inline const Mat& operator*=(const subview_elem2<eT,T1,T2>& X);
   template<typename T1, typename T2> inline const Mat& operator%=(const subview_elem2<eT,T1,T2>& X);
   template<typename T1, typename T2> inline const Mat& operator/=(const subview_elem2<eT,T1,T2>& X);
-
-  // Operators on sparse matrices (and subviews).
+  
+  // Operators on sparse matrices (and subviews)
   template<typename T1> inline explicit          Mat(const SpBase<eT, T1>& m);
   template<typename T1> inline const Mat&  operator=(const SpBase<eT, T1>& m);
   template<typename T1> inline const Mat& operator+=(const SpBase<eT, T1>& m);
@@ -201,6 +218,18 @@ class Mat : public Base< eT, Mat<eT> >
   inline            subview<eT> operator()(const uword in_row1, const uword in_col1, const SizeMat& s);
   inline      const subview<eT> operator()(const uword in_row1, const uword in_col1, const SizeMat& s) const;
   
+  inline       subview<eT> head_rows(const uword N);
+  inline const subview<eT> head_rows(const uword N) const;
+  
+  inline       subview<eT> tail_rows(const uword N);
+  inline const subview<eT> tail_rows(const uword N) const;
+  
+  inline       subview<eT> head_cols(const uword N);
+  inline const subview<eT> head_cols(const uword N) const;
+  
+  inline       subview<eT> tail_cols(const uword N);
+  inline const subview<eT> tail_cols(const uword N) const;
+  
   template<typename T1> arma_inline       subview_elem1<eT,T1> elem(const Base<uword,T1>& a);
   template<typename T1> arma_inline const subview_elem1<eT,T1> elem(const Base<uword,T1>& a) const;
   
@@ -225,11 +254,26 @@ class Mat : public Base< eT, Mat<eT> >
   template<typename T2> arma_inline const subview_elem2<eT,T2,T2> cols(const Base<uword,T2>& ci) const;
   
   
-  arma_inline subview_each1< Mat<eT>, 0 > each_col();
-  arma_inline subview_each1< Mat<eT>, 1 > each_row();
+  arma_inline       subview_each1< Mat<eT>, 0 > each_col();
+  arma_inline       subview_each1< Mat<eT>, 1 > each_row();
+  
+  arma_inline const subview_each1< Mat<eT>, 0 > each_col() const;
+  arma_inline const subview_each1< Mat<eT>, 1 > each_row() const;
+  
+  template<typename T1> inline       subview_each2< Mat<eT>, 0, T1 > each_col(const Base<uword, T1>& indices);
+  template<typename T1> inline       subview_each2< Mat<eT>, 1, T1 > each_row(const Base<uword, T1>& indices);
+  
+  template<typename T1> inline const subview_each2< Mat<eT>, 0, T1 > each_col(const Base<uword, T1>& indices) const;
+  template<typename T1> inline const subview_each2< Mat<eT>, 1, T1 > each_row(const Base<uword, T1>& indices) const;
+  
+  #if defined(ARMA_USE_CXX11)
+  inline const Mat& each_col(const std::function< void(      Col<eT>&) >& F);
+  inline const Mat& each_col(const std::function< void(const Col<eT>&) >& F) const;
+  
+  inline const Mat& each_row(const std::function< void(      Row<eT>&) >& F);
+  inline const Mat& each_row(const std::function< void(const Row<eT>&) >& F) const;
+  #endif
   
-  template<typename T1> inline subview_each2< Mat<eT>, 0, T1 > each_col(const Base<uword, T1>& indices);
-  template<typename T1> inline subview_each2< Mat<eT>, 1, T1 > each_row(const Base<uword, T1>& indices);
   
   arma_inline       diagview<eT> diag(const sword in_id = 0);
   arma_inline const diagview<eT> diag(const sword in_id = 0) const;
@@ -338,6 +382,12 @@ class Mat : public Base< eT, Mat<eT> >
   arma_inline arma_warn_unused bool is_square() const;
        inline arma_warn_unused bool is_finite() const;
   
+  inline arma_warn_unused bool has_inf() const;
+  inline arma_warn_unused bool has_nan() const;
+  
+  inline arma_warn_unused bool is_sorted(const char* direction = "ascend")       const;
+  inline arma_warn_unused bool is_sorted(const char* direction, const uword dim) const;
+  
   arma_inline arma_warn_unused bool in_range(const uword ii) const;
   arma_inline arma_warn_unused bool in_range(const span& x ) const;
   
@@ -355,10 +405,10 @@ class Mat : public Base< eT, Mat<eT> >
   arma_inline arma_warn_unused const eT* memptr() const;
   
   
-  inline void impl_print(const std::string& extra_text) const;
+  inline void impl_print(                           const std::string& extra_text) const;
   inline void impl_print(std::ostream& user_stream, const std::string& extra_text) const;
   
-  inline void impl_raw_print(const std::string& extra_text) const;
+  inline void impl_raw_print(                           const std::string& extra_text) const;
   inline void impl_raw_print(std::ostream& user_stream, const std::string& extra_text) const;
   
   
@@ -367,18 +417,23 @@ class Mat : public Base< eT, Mat<eT> >
   
   inline void set_size(const uword in_elem);
   inline void set_size(const uword in_rows, const uword in_cols);
+  inline void set_size(const SizeMat& s);
   
   inline void   resize(const uword in_elem);
   inline void   resize(const uword in_rows, const uword in_cols);
+  inline void   resize(const SizeMat& s);
   
-  inline void  reshape(const uword in_rows, const uword in_cols, const uword dim = 0);
+  inline void  reshape(const uword in_rows, const uword in_cols);
+  inline void  reshape(const SizeMat& s);
+  
+  arma_deprecated inline void reshape(const uword in_rows, const uword in_cols, const uword dim);  //!< NOTE: don't use this form: it's deprecated
   
   
-  template<typename functor>
-  inline const Mat& transform(functor F);
+  template<typename functor> inline const Mat&  for_each(functor F);
+  template<typename functor> inline const Mat&  for_each(functor F) const;
   
-  template<typename functor>
-  inline const Mat& imbue(functor F);
+  template<typename functor> inline const Mat& transform(functor F);
+  template<typename functor> inline const Mat&     imbue(functor F);
   
   
   arma_hot inline const Mat& fill(const eT val);
@@ -389,21 +444,26 @@ class Mat : public Base< eT, Mat<eT> >
   inline const Mat& zeros();
   inline const Mat& zeros(const uword in_elem);
   inline const Mat& zeros(const uword in_rows, const uword in_cols);
+  inline const Mat& zeros(const SizeMat& s);
   
   inline const Mat& ones();
   inline const Mat& ones(const uword in_elem);
   inline const Mat& ones(const uword in_rows, const uword in_cols);
+  inline const Mat& ones(const SizeMat& s);
   
   inline const Mat& randu();
   inline const Mat& randu(const uword in_elem);
   inline const Mat& randu(const uword in_rows, const uword in_cols);
+  inline const Mat& randu(const SizeMat& s);
   
   inline const Mat& randn();
   inline const Mat& randn(const uword in_elem);
   inline const Mat& randn(const uword in_rows, const uword in_cols);
+  inline const Mat& randn(const SizeMat& s);
   
   inline const Mat& eye();
   inline const Mat& eye(const uword in_rows, const uword in_cols);
+  inline const Mat& eye(const SizeMat& s);
   
   inline void reset();
   
@@ -492,6 +552,86 @@ class Mat : public Base< eT, Mat<eT> >
     arma_aligned       uword    col;
     };
   
+  
+  class const_row_col_iterator;
+
+  class row_col_iterator
+    {
+    public:
+    
+    inline row_col_iterator();
+    inline row_col_iterator(const row_col_iterator& in_it);
+    inline row_col_iterator(Mat<eT>& in_M, const uword row = 0, const uword col = 0);
+    
+    inline arma_hot eT& operator*();
+    
+    inline arma_hot row_col_iterator& operator++();
+    inline arma_hot row_col_iterator  operator++(int);
+    inline arma_hot row_col_iterator& operator--();
+    inline arma_hot row_col_iterator  operator--(int);
+    
+    inline uword row() const;
+    inline uword col() const;
+    
+    inline arma_hot bool operator==(const       row_col_iterator& rhs) const;
+    inline arma_hot bool operator!=(const       row_col_iterator& rhs) const;
+    inline arma_hot bool operator==(const const_row_col_iterator& rhs) const;
+    inline arma_hot bool operator!=(const const_row_col_iterator& rhs) const;
+    
+    // So that we satisfy the STL iterator types.
+    typedef std::bidirectional_iterator_tag iterator_category;
+    typedef eT                              value_type;
+    typedef uword                           difference_type; // not certain on this one
+    typedef const eT*                       pointer;
+    typedef const eT&                       reference;
+    
+    arma_aligned Mat<eT>* M;
+    
+    arma_aligned eT*    current_pos;
+    arma_aligned uword  internal_col;
+    arma_aligned uword  internal_row;
+    };
+  
+  
+  class const_row_col_iterator
+    {
+    public:
+    
+    inline const_row_col_iterator();
+    inline const_row_col_iterator(const       row_col_iterator& in_it);
+    inline const_row_col_iterator(const const_row_col_iterator& in_it);
+    inline const_row_col_iterator(const Mat<eT>& in_M, const uword row = 0, const uword col = 0);
+    
+    inline arma_hot const eT& operator*() const;
+    
+    inline arma_hot const_row_col_iterator& operator++();
+    inline arma_hot const_row_col_iterator  operator++(int);
+    inline arma_hot const_row_col_iterator& operator--();
+    inline arma_hot const_row_col_iterator  operator--(int);
+    
+    inline uword row() const;
+    inline uword col() const;
+    
+    inline arma_hot bool operator==(const const_row_col_iterator& rhs) const;
+    inline arma_hot bool operator!=(const const_row_col_iterator& rhs) const;
+    inline arma_hot bool operator==(const       row_col_iterator& rhs) const;
+    inline arma_hot bool operator!=(const       row_col_iterator& rhs) const;
+    
+    // So that we satisfy the STL iterator types.
+    typedef std::bidirectional_iterator_tag iterator_category;
+    typedef eT                              value_type;
+    typedef uword                           difference_type; // not certain on this one
+    typedef const eT*                       pointer;
+    typedef const eT&                       reference;
+    
+    arma_aligned const Mat<eT>* M;
+    
+    arma_aligned const eT*    current_pos;
+    arma_aligned       uword  internal_col;
+    arma_aligned       uword  internal_row;
+    };
+  
+  
   inline       iterator  begin();
   inline const_iterator  begin() const;
   inline const_iterator cbegin() const;
@@ -512,6 +652,13 @@ class Mat : public Base< eT, Mat<eT> >
   inline       row_iterator end_row  (const uword row_num);
   inline const_row_iterator end_row  (const uword row_num) const;
   
+  inline       row_col_iterator begin_row_col();
+  inline const_row_col_iterator begin_row_col() const;
+  
+  inline       row_col_iterator end_row_col();
+  inline const_row_col_iterator end_row_col() const;
+  
+  
   inline void  clear();
   inline bool  empty() const;
   inline uword size()  const;
@@ -520,6 +667,9 @@ class Mat : public Base< eT, Mat<eT> >
   
   inline void steal_mem(Mat& X);  //!< don't use this unless you're writing code internal to Armadillo
   
+  inline void steal_mem_col(Mat& X, const uword max_n_rows);
+  
+  
   template<uword fixed_n_rows, uword fixed_n_cols> class fixed;
   
   
@@ -531,7 +681,8 @@ class Mat : public Base< eT, Mat<eT> >
   inline void init(const std::string& text);
   
   #if defined(ARMA_USE_CXX11)
-  inline void init(const std::initializer_list<eT>& list);
+    inline void init(const std::initializer_list<eT>& list);
+    inline void init(const std::initializer_list< std::initializer_list<eT> >& list);
   #endif
   
   template<typename T1, typename T2>
@@ -546,6 +697,7 @@ class Mat : public Base< eT, Mat<eT> >
   
   
   friend class Cube<eT>;
+  friend class subview_cube<eT>;
   friend class glue_join;
   friend class op_strans;
   friend class op_htrans;
@@ -604,10 +756,18 @@ class Mat<eT>::fixed : public Mat<eT>
   #if defined(ARMA_USE_CXX11)
     inline                fixed(const std::initializer_list<eT>& list);
     inline const Mat& operator=(const std::initializer_list<eT>& list);
+    
+    inline                fixed(const std::initializer_list< std::initializer_list<eT> >& list);
+    inline const Mat& operator=(const std::initializer_list< std::initializer_list<eT> >& list);
   #endif
   
   arma_inline const Mat& operator=(const fixed<fixed_n_rows, fixed_n_cols>& X);
   
+  #if defined(ARMA_GOOD_COMPILER)
+    template<typename T1,              typename   eop_type> inline const Mat& operator=(const   eOp<T1,       eop_type>& X);
+    template<typename T1, typename T2, typename eglue_type> inline const Mat& operator=(const eGlue<T1, T2, eglue_type>& X);
+  #endif
+  
   arma_inline const Op< Mat_fixed_type, op_htrans >  t() const;
   arma_inline const Op< Mat_fixed_type, op_htrans > ht() const;
   arma_inline const Op< Mat_fixed_type, op_strans > st() const;
