@@ -1,8 +1,11 @@
-// Copyright (C) 2013 Conrad Sanderson
+// Copyright (C) 2013 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup xvec_htrans
@@ -33,6 +36,8 @@ class xvec_htrans : public Base<eT, xvec_htrans<eT> >
   
   inline eT operator[](const uword ii) const;
   inline eT at_alt    (const uword ii) const;
+  
+  inline eT at        (const uword in_row, const uword in_col) const;
   };
 
 
